#include <qthing.h>
#include "event.h"

string_callback_t log_oled_handler = NULL;
void set_oled_log_handler(string_callback_t handler) {
    log_oled_handler = handler;
}
void log_oled(const std::string& message) {
    if (log_oled_handler != NULL) {
        log_oled_handler(message);
    }
}

connection_status_t wlan_connection_status = uninitialized;
connection_status_callback_t wlan_connection_status_handler = NULL;
void add_wlan_connection_status_handler(connection_status_callback_t handler) {
    if (wlan_connection_status_handler == NULL) {
        wlan_connection_status_handler = handler;
    }
    else {
        wlan_connection_status_handler = [wlan_connection_status_handler, handler](connection_status_t status){
            wlan_connection_status_handler(status);
            handler(status);
        };
    }
}
void update_wlan_connection_status(connection_status_t status) {
    wlan_connection_status = status;
    if (wlan_connection_status_handler != NULL) {
        wlan_connection_status_handler(status);
    }
}
connection_status_t get_wlan_connection_status() {
    return wlan_connection_status;
}

connection_status_t mqtt_connection_status = uninitialized;
connection_status_callback_t mqtt_connection_status_handler = NULL;
void add_mqtt_connection_status_handler(connection_status_callback_t handler) {
    if (mqtt_connection_status_handler == NULL) {
        mqtt_connection_status_handler = handler;
    }
    else {
        mqtt_connection_status_handler = [mqtt_connection_status_handler, handler](connection_status_t status){
            mqtt_connection_status_handler(status);
            handler(status);
        };
    }
}
void update_mqtt_connection_status(connection_status_t status) {
    mqtt_connection_status = status;
    if (mqtt_connection_status_handler != NULL) {
        mqtt_connection_status_handler(status);
    }
}
connection_status_t get_mqtt_connection_status() {
    return mqtt_connection_status;
}

connection_status_callback_t combined_mqtt_connection_status_handler = NULL;
void update_combined_mqtt_connection_status() {
    connection_status_t status = uninitialized;
    if (wlan_connection_status == uninitialized) {
        status = uninitialized;
    }
    else if (wlan_connection_status == disconnected) {
        status = disconnected;
    }
    else if (wlan_connection_status == connecting || mqtt_connection_status == connecting) {
        status = connecting;
    }
    else if (mqtt_connection_status == disconnected) {
        status = disconnected;
    }
    else if (wlan_connection_status == connected && mqtt_connection_status == connected) {
        status = connected;
    }
    combined_mqtt_connection_status_handler(status);
}
void add_combined_mqtt_connection_status_handler(connection_status_callback_t handler) {
    if (combined_mqtt_connection_status_handler == NULL) {
        combined_mqtt_connection_status_handler = handler;
        add_wlan_connection_status_handler([](connection_status_t wlan_status){
            update_combined_mqtt_connection_status();
        });
        add_mqtt_connection_status_handler([](connection_status_t mqtt_status){
            update_combined_mqtt_connection_status();
        });
    }
    else {
        combined_mqtt_connection_status_handler = [combined_mqtt_connection_status_handler, handler](connection_status_t status){
            combined_mqtt_connection_status_handler(status);
            handler(status);
        };
    }
}
